/*
 * cimrpmv4.c
 *
 * (C) Copyright IBM Corp. 2002
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors:
 *
 * Description: RPM V4 services for CIM
 */

#include "cimrpm.h"
#include "cimrpmfp.h"
#include <OSBase_Common.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/user.h>

/* Enumeration Helper Structure and eye catcher (MAGIC) */

#define EH_MAGIC 0x17711881

#define	PAGE_SIZE (sysconf(_SC_PAGESIZE))

typedef struct {
  long     ehMagic;
  char   **ehListResult;
  size_t   ehListIndex;
  char    *ehCmdstr;
} EnumHelper;

/* helper function - convert header to software element */
static int line2se(char *, CRPM_SE*);
static char * makecmd(const char *, const char *);
static int InitPkgHelper(HCRPM,CRPM_SE*, const char *);

/* Does nothing  */
HCRPM CRPM_Initv4()
{
  HCRPM dbptr = (HCRPM)0xdeadbeef;
  return dbptr;
}

/* Does nothing */
void CRPM_Termv4(HCRPM dbptr)
{
}

/* Set up a new package enumeration */
int CRPM_InitPkgEnumv4(HCRPM dbptr,CRPM_SE* se)
{
  return InitPkgHelper(dbptr,se,"--all");
}

int CRPM_NextPkgEnumv4(CRPM_SE* se)
{
  EnumHelper* eh;

  if (se==NULL || se->reserved==NULL)
    return -1;
  
  eh=se->reserved;
  if (eh->ehListResult == NULL || eh->ehListResult[eh->ehListIndex]==NULL)
    return 0;
  else {
    line2se(eh->ehListResult[eh->ehListIndex],se);
    eh->ehListIndex+=1;
    return 1;
  }

}

/* Terminate package enumeration and free allocated memory */
void CRPM_TermPkgEnumv4(CRPM_SE* se)
{
  EnumHelper * eh;

  if (se) {
    eh=se->reserved;
    /* ptr validity check -- see InitPkgEnum */
    if ((int)eh > PAGE_SIZE && (void*)eh < sbrk(0) && eh->ehMagic==EH_MAGIC) {
      if (eh->ehCmdstr)
	free(eh->ehCmdstr);
      if (eh->ehListResult)
	freeresultbuf(eh->ehListResult);
      free(eh);
      se->reserved=NULL;
    }
  }
}

/* Retrieve single software element */
int CRPM_SinglePkgEnumv4(HCRPM hcrpm,const char * spec,CRPM_SE* se)
{
  int rc;
  rc = InitPkgHelper(hcrpm,se,spec);
  if (CRPM_NextPkgEnumv4(se))
    rc=0;
  else
    rc=-1;
  return rc;
}

/* Retrieve list of files for software element */
char ** CRPM_SinglePkgFilesv4(HCRPM hcrpm, const char* spec, CRPM_SE* se)
{
  char ** filelist = NULL;
  char *  cmdstr;
  int     rc;

  cmdstr=makecmd("rpm --query --list ",
		 spec);
  rc=runcommand(cmdstr,NULL,&filelist,NULL);
  free (cmdstr);

  if (rc && filelist) {
    freeresultbuf(filelist);
    filelist=NULL;
  }
  
  return filelist;
}


/* enumeration helper */
static int InitPkgHelper(HCRPM dbptr,CRPM_SE* se, const char * spec)
{
  EnumHelper* eh;
  int rc;

  if (se==NULL || dbptr==NULL)
    return -1;
  
  eh=se->reserved;
  /* we are checking that pointer is valid by making sure
     it is pointing to memory above the first page and below the
     address space break value */
  if ((int)eh >= PAGE_SIZE && (void*)eh < sbrk(0) && eh->ehMagic==EH_MAGIC) {
    /* leftover from previous enumeration */
    CRPM_TermPkgEnumv4(se);
  }
  
  /* clean the software element structure */
  memset(se,0,sizeof(CRPM_SE));

  se->reserved=malloc(sizeof(EnumHelper));
  if (se->reserved==NULL)
    return -1;
  
  eh=se->reserved;
  eh->ehMagic=EH_MAGIC;
  eh->ehListResult=NULL;
  eh->ehListIndex=0;

  /* enumerate the instances */
  eh->ehCmdstr=makecmd("rpm --query --queryformat="
		       "%{NAME}%%%{VERSION}%%%{PACKAGER}"
		       "%%%{SUMMARY}%%%{INSTALLTIME}%%%{OS}%%\\\\n ",
		       spec);
  if (eh->ehCmdstr==NULL) {
    CRPM_TermPkgEnumv4(se);
    return -1;
  }
  rc = runcommand(eh->ehCmdstr,NULL,&(eh->ehListResult),NULL);
  
  return rc;
}

static int line2se(char * line, CRPM_SE* se)
{
  char * dupline, *helper, *token;

  if (line==NULL)
    return -1;
  
  /* free the previous elements */
  if(se->seName) free(se->seName);
  if(se->seId) free(se->seId);
  if(se->seVersion) free(se->seVersion);
  if(se->seManufacturer) free(se->seManufacturer);
  if(se->seInstalled) free(se->seInstalled);
  if(se->seCaption) free(se->seCaption);
  if(se->seDescription) free(se->seDescription);
  
  /* clear the user part of the structure */
  memset(se,0,(size_t)&(((CRPM_SE*)NULL)->reserved));
  
  dupline=strdup(line);
  /* direct values */
  token=strtok_r(dupline,"%",&helper);
  if (token) {
    se->seName=strdup(token);
    se->seId=strdup(token);
  }
  token=strtok_r(NULL,"%",&helper);
  if (token)
    se->seVersion=strdup(token);
  token=strtok_r(NULL,"%",&helper);
  if (token)
    se->seManufacturer=strdup(token);
  token=strtok_r(NULL,"%",&helper);
  if (token)
    se->seCaption=strdup(token);
  /** description - not yet working
      token=strtok_r(NULL,"%",&helper);
      if (token)
      se->seDescription=strdup(token);
  */
  token=strtok_r(NULL,"%",&helper);
  if (token)
    se->seInstalled=strdup(token);
  /* mapped values */
  token=strtok_r(NULL,"%",&helper);
  if (token && strncasecmp("linux",token,5)==0)
    se->seOs=36; /* Linux, if counted correctly */
  else
    se->seOs=0;
  /* fixed values */
  se->seState=2; /* installed */
  
  free(dupline);
  return 0;
}

static char * makecmd(const char * prog, const char * arg)
{
  char * str = malloc(strlen(prog) + strlen(arg) + 2);
  strcpy(str,prog);
  strcat(str," ");
  strcat(str,arg);
  return str;
}

/* Initialization stuff */
struct _FPCRPM FP_CRPM;

void _init()
{
  /* hi */
  fprintf(stderr,"loading libcimrpmv4.so\n");
  
  FP_CRPM.fpCRPM_Init = CRPM_Initv4;
  FP_CRPM.fpCRPM_Term = CRPM_Termv4;
  FP_CRPM.fpCRPM_InitPkgEnum = CRPM_InitPkgEnumv4;
  FP_CRPM.fpCRPM_NextPkgEnum = CRPM_NextPkgEnumv4;
  FP_CRPM.fpCRPM_TermPkgEnum = CRPM_TermPkgEnumv4;
  FP_CRPM.fpCRPM_SinglePkgEnum = CRPM_SinglePkgEnumv4;
  FP_CRPM.fpCRPM_SinglePkgFiles = CRPM_SinglePkgFilesv4;
}

void _fini()
{
  /* bye */
}
